@@ -1,27 +1,30 @@
 from pygame import Surface, Rect
 
-from typing import Iterable, List, Tuple, Optional, FrozenSet, Any
+from typing import List, Tuple, Optional, FrozenSet, Any
 from weakref import ref
 
 
 class Recurface:
     def __init__(
-            self, surface: Optional[Surface] = None, parent: Optional["Recurface"] = None,
-            position: Optional[Tuple[float, float]] = None, priority: Any = None):
-        self.__surface = surface  # Must hold a valid pygame Surface in order to successfully render
+            self, surface: Optional[Surface] = None, position: Optional[Tuple[float, float]] = None,
+            parent: Optional["Recurface"] = None, priority: Any = None
+    ):
         self.__render_position = list(position) if position else None  # (x, y) to render at in the containing Surface
         self.__render_priority = priority  # Determines how recurfaces at the same nesting level are layered on screen
 
         self.__rect = None
         self.__rect_previous = None
-        self.__rect_additional = []
+        self.__additional_rects = []
 
         self.__child_recurfaces = set()
         self.__frozen_child_recurfaces = frozenset()
         self.__ordered_child_recurfaces = tuple()
 
+        self.__surface = None  # Must hold a valid pygame Surface in order to successfully render
+        self.surface = surface  # Deliberately invokes the code in .surface's setter
+
         self.__parent_recurface = None
-        self.parent_recurface = parent  # Done this way to invoke the code in .parent's setter
+        self.parent_recurface = parent  # Deliberately invokes the code in .parent's setter
 
     @property
     def surface(self) -> Surface:
@@ -29,11 +32,11 @@ class Recurface:
 
     @surface.setter
     def surface(self, value: Surface):
-        if self.__surface is value:
+        if self.__surface == value:
             return  # Surface is already correctly set
 
-        self.__rect_previous = self.__rect
         self.__surface = value
+        self.update_surface()
 
     @property
     def render_position(self) -> Optional[Tuple[float, float]]:
@@ -49,45 +52,45 @@ class Recurface:
                 return  # Position is already correctly set
 
         if self.__render_position:
-            self.__rect_previous = self.__rect
+            self.update_surface()
 
         self.__render_position = [value[0], value[1]] if value else None
 
     @property
     def x_render_position(self) -> float:
         if self.__render_position is None:
-            raise ValueError(".render_position is not currently set")
+            raise ValueError("`.render_position` is not currently set")
 
         return self.__render_position[0]
 
     @x_render_position.setter
     def x_render_position(self, value: float):
         if self.__render_position is None:
-            raise ValueError(".render_position is not currently set")
+            raise ValueError("`.render_position` is not currently set")
 
         if self.__render_position[0] == value:
             return  # Position is already correctly set
 
-        self.__rect_previous = self.__rect
         self.__render_position[0] = value
+        self.update_surface()
 
     @property
     def y_render_position(self) -> float:
         if self.__render_position is None:
-            raise ValueError(".render_position is not currently set")
+            raise ValueError("`.render_position` is not currently set")
 
         return self.__render_position[1]
 
     @y_render_position.setter
     def y_render_position(self, value: float):
         if self.__render_position is None:
-            raise ValueError(".render_position is not currently set")
+            raise ValueError("`.render_position` is not currently set")
 
         if self.__render_position[1] == value:
             return  # Position is already correctly set
 
-        self.__rect_previous = self.__rect
         self.__render_position[1] = value
+        self.update_surface()
 
     @property
     def parent_recurface(self) -> Optional["Recurface"]:
@@ -104,7 +107,7 @@ class Recurface:
             if curr_parent is value:
                 return  # Parent is already correctly set
 
-            self._reset_rects(forward_rects=True)
+            self._reset_rects(do_forward_rects=True)
             curr_parent.remove_child_recurface(self)  # Remove from any previous parent
 
         self.__parent_recurface = None if value is None else ref(value)
@@ -137,6 +140,9 @@ class Recurface:
 
     @render_priority.setter
     def render_priority(self, value: Any):
+        if self.__render_priority == value:
+            return  # Priority is already correctly set
+
         self.__render_priority = value
 
         if self.parent_recurface:
@@ -151,7 +157,7 @@ class Recurface:
 
         child.parent_recurface = self
 
-        child._reset_rects()  # Extra call to reset() for redundancy
+        child._reset_rects()  # Resetting child rects here for redundancy
 
     def remove_child_recurface(self, child: "Recurface") -> None:
         if child in self.__child_recurfaces:
@@ -173,11 +179,24 @@ class Recurface:
 
         return self.render_position
 
-    def add_update_rects(self, rects: Iterable[Optional[Rect]], update_position: bool = False) -> None:
+    def update_surface(self) -> None:
         """
-        Stores the provided pygame rects to be returned by this recurface on the next render() call.
+        This method manually flags the stored surface's area to be updated on the next render.
+        It should be invoked externally whenever the stored surface has been mutated in place
+        (as opposed to replaced with a different Surface object)
+        """
+
+        self.__rect_previous = self.__rect
+
+    def add_update_rects(self, *rects: Optional[Rect], do_update_position: bool = False) -> None:
+        """
+        Stores the provided pygame rects to be returned by this recurface on the next `.render()` call.
         Used internally to handle removing child objects.
-        If update_position is True, the provided rects will be offset by the position of .__rect before storing.
+
+        Any rects passed into this method should be bounded within the area of this recurface's stored surface,
+        as they will only be used if not all of this recurface's surface area needs updating on the next render.
+
+        If do_update_position is True, the provided rects will be offset by the position of .__rect before storing
         """
 
         is_rendered = bool(self.__rect)  # If area has been rendered previously
@@ -186,16 +205,16 @@ class Recurface:
 
         for rect in rects:
             if rect:
-                if update_position:
+                if do_update_position:
                     """
                     Assumes that the area each provided rect represents is offset from the *last rendered* position
-                    of this recurface's rect, not the *current* position - as would be the case if the provided rects
-                    are from now-removed child recurfaces
+                    of this recurface's rect, not the *current* position - as the former would be the case if
+                    the provided rects are from now-removed child recurfaces
                     """
                     rect.x += self.__rect.x
                     rect.y += self.__rect.y
 
-                self.__rect_additional.append(rect)
+                self.__additional_rects.append(rect)
 
     def render(self, destination: Surface) -> List[Rect]:
         """
@@ -211,7 +230,7 @@ class Recurface:
     def _render(self, destination: Surface) -> List[Optional[Rect]]:
         """
         Draws all child surfaces to a copy of .surface, then draws the copy to the provided destination.
-        Returns a list of pygame rects representing updated areas of the provided destination.
+        Returns a list of pygame rects representing updated areas of the provided destination
         """
 
         result = []
@@ -225,8 +244,8 @@ class Recurface:
             return result
 
         if self.surface is None:
-            raise ValueError(".surface does not contain a valid pygame Surface to render")
-        surface_working = self.surface.copy()
+            raise ValueError("`.surface` does not contain a valid pygame Surface to render")
+        surface_working = self.copy_surface()
 
         try:
             child_recurfaces = self.ordered_child_recurfaces
@@ -258,12 +277,12 @@ class Recurface:
         else:  # Child and additional rects are only used if the full area was not updated
             result += child_rects
 
-            if self.__rect_additional:  # If there are any extra areas that need updating
-                result += self.__rect_additional
+            if self.__additional_rects:  # If there are any extra areas that need updating
+                result += self.__additional_rects
 
         # Only .__rect should retain its value post-render. Whether used or not, ._previous and ._additional are reset
         self.__rect_previous = None
-        self.__rect_additional = []
+        self.__additional_rects = []
         return result
 
     def unlink(self) -> None:
@@ -280,20 +299,31 @@ class Recurface:
             child.move_render_position(*self.render_position)
             child.parent_recurface = parent
 
-        self._reset_rects()
+        self._reset_rects()  # Resetting rects here for redundancy
 
-    def _reset_rects(self, forward_rects: bool = False) -> None:
+    def copy_surface(self) -> Surface:
+        """
+        Can optionally be overridden.
+
+        Generates a copy of this Recurface object's stored Surface. Uses the standard `Surface.copy()` method
+        provided by pygame by default, but if there is a less resource-intensive process to do so for
+        any particular subclass then that process may be applied by overriding this method
+        """
+
+        return self.surface.copy()
+
+    def _reset_rects(self, do_forward_rects: bool = False) -> None:
         """
         Sets variables which hold the object's rendering details back to their default values.
-        This should typically only be done if the parent object is being changed
+        This is automatically done whenever the parent object is changed
         """
 
-        if forward_rects and self.parent_recurface:
-            self.parent_recurface.add_update_rects([self.__rect], update_position=True)
+        if do_forward_rects and self.parent_recurface:
+            self.parent_recurface.add_update_rects(self.__rect, do_update_position=True)
 
         self.__rect = None
         self.__rect_previous = None
-        self.__rect_additional = []
+        self.__additional_rects = []
 
     def _organise_child_recurfaces(self) -> None:
         self.__frozen_child_recurfaces = frozenset(self.__child_recurfaces)
@@ -310,12 +340,14 @@ class Recurface:
         """
         Optimisation method applied in `.render_to_display()`.
 
-        Trims any None objects from the provided list, and any Rect objects whose bounds are entirely contained
-        within another Rect object in the list
+        Returns a new list containing the same rects as the provided list, but with any None objects removed
+        and any Rect objects whose bounds are entirely contained within another Rect object in the list also removed
+        (this includes removing additional identical copies of rects)
         """
 
         result = []
 
+        excluded_rect_indexes = set()
         for rect_index, rect in enumerate(rects):
             if rect is None:
                 continue
@@ -328,16 +360,24 @@ class Recurface:
                 if other_rect_index == rect_index:
                     continue
 
-                if is_rect_inside_other_rect := (
-                    (rect.left >= other_rect.left) and
-                    (rect.right <= other_rect.right) and
-                    (rect.top >= other_rect.top) and
-                    (rect.bottom >= other_rect.bottom)
+                if other_rect_index in excluded_rect_indexes:
+                    continue
+
+                if is_rect_outside_other_rect := (
+                    (rect.left < other_rect.left) or
+                    (rect.right > other_rect.right) or
+                    (rect.top < other_rect.top) or
+                    (rect.bottom > other_rect.bottom)
                 ):
+                    continue
+
+                else:  # If `rect` is equal to or inside `other_rect`
                     include_rect = False
                     break
 
             if include_rect:
                 result.append(rect)
+            else:
+                excluded_rect_indexes.add(rect_index)
 
         return result
